@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2007 The GNOME Foundation
  * Written by Thomas Wood <thos@gnome.org>
+ *            Jens Granseuer <jensgr@gmx.net>
  * All Rights Reserved
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,33 +25,141 @@
 #include "theme-installer.h"
 #include "theme-save.h"
 #include "theme-util.h"
+#include "gtkrc-utils.h"
+#include "gedit-message-area.h"
+#include "wp-cellrenderer.h"
 
 #include <glib/gi18n.h>
 #include <libwindow-settings/gnome-wm-manager.h>
 #include <string.h>
+#include <libgnomevfs/gnome-vfs.h>
+#include <libgnomeui/gnome-thumbnail.h>
 
 #define CUSTOM_THEME_NAME "__custom__"
 
+enum
+{
+  RESPONSE_APPLY_BG,
+  RESPONSE_APPLY_FONT
+};
+
+enum
+{
+  TARGET_URI_LIST,
+  TARGET_NS_URL
+};
+
+static const GtkTargetEntry drop_types[] =
+{
+  {"text/uri-list", 0, TARGET_URI_LIST},
+  {"_NETSCAPE_URL", 0, TARGET_NS_URL}
+};
+
+static time_t
+theme_get_mtime (const char *name)
+{
+  GnomeThemeMetaInfo *theme;
+  time_t mtime = -1;
+
+  theme = gnome_theme_meta_info_find (name);
+  if (theme != NULL) {
+    GnomeVFSFileInfo *file_info;
+    GnomeVFSResult result;
+
+    file_info = gnome_vfs_file_info_new ();
+
+    result = gnome_vfs_get_file_info (theme->path, file_info,
+				      GNOME_VFS_FILE_INFO_DEFAULT |
+				      GNOME_VFS_FILE_INFO_FOLLOW_LINKS);
+
+    if (result == GNOME_VFS_OK)
+      mtime = file_info->mtime;
+
+    gnome_vfs_file_info_unref (file_info);
+  }
+
+  return mtime;
+}
+
+static void
+theme_thumbnail_update (GdkPixbuf *pixbuf,
+			gchar *theme_name,
+			AppearanceData *data,
+			gboolean cache)
+{
+  GtkTreeIter iter;
+  GtkTreeModel *model = GTK_TREE_MODEL (data->theme_store);
+
+  /* find item in model and update thumbnail */
+  if (!pixbuf)
+    return;
+
+  if (theme_find_in_model (model, theme_name, &iter)) {
+    time_t mtime;
+
+    gtk_list_store_set (data->theme_store, &iter, COL_THUMBNAIL, pixbuf, -1);
+
+    /* cache thumbnail */
+    if (cache && (mtime = theme_get_mtime (theme_name)) != -1) {
+      gchar *path;
+
+      /* try to share thumbs with nautilus, use themes:/// */
+      path = g_strconcat ("themes:///", theme_name, NULL);
+
+      gnome_thumbnail_factory_save_thumbnail (data->thumb_factory,
+					      pixbuf, path, mtime);
+
+      g_free (path);
+    }
+  }
+
+  g_object_unref (pixbuf);
+}
+
+static GdkPixbuf *
+theme_get_thumbnail_from_cache (GnomeThemeMetaInfo *info, AppearanceData *data)
+{
+  GdkPixbuf *thumb = NULL;
+  gchar *path, *thumb_filename;
+  time_t mtime;
+
+  if (info == data->theme_custom)
+    return NULL;
+
+  mtime = theme_get_mtime (info->name);
+  if (mtime == -1)
+    return NULL;
+
+  /* try to share thumbs with nautilus, use themes:/// */
+  path = g_strconcat ("themes:///", info->name, NULL);
+  thumb_filename = gnome_thumbnail_factory_lookup (data->thumb_factory,
+                                                   path, mtime);
+  g_free (path);
+
+  if (thumb_filename != NULL)
+    thumb = gdk_pixbuf_new_from_file (thumb_filename, NULL);
+
+  return thumb;
+}
+
 static void
 theme_thumbnail_done_cb (GdkPixbuf *pixbuf, gchar *theme_name, AppearanceData *data)
 {
-  /* find item in model and update thumbnail */
-  if (pixbuf) {
-    GtkTreeIter iter;
-    GtkTreeModel *model = GTK_TREE_MODEL (data->theme_store);
-
-    if (theme_find_in_model (model, theme_name, &iter))
-      gtk_list_store_set (data->theme_store, &iter, COL_THUMBNAIL, pixbuf, -1);
-
-    g_object_unref (pixbuf);
-  }
+  theme_thumbnail_update (pixbuf, theme_name, data, TRUE);
 }
 
 static void
 theme_thumbnail_generate (GnomeThemeMetaInfo *info, AppearanceData *data)
 {
-  generate_meta_theme_thumbnail_async (info,
-      (ThemeThumbnailFunc) theme_thumbnail_done_cb, data, NULL);
+  GdkPixbuf *thumb;
+
+  thumb = theme_get_thumbnail_from_cache (info, data);
+
+  if (thumb != NULL)
+    theme_thumbnail_update (thumb, info->name, data, FALSE);
+  else
+    generate_meta_theme_thumbnail_async (info,
+        (ThemeThumbnailFunc) theme_thumbnail_done_cb, data, NULL);
 }
 
 static void
@@ -110,17 +219,34 @@ is_locked_down (GConfClient *client)
 static void
 theme_load_from_gconf (GConfClient *client, GnomeThemeMetaInfo *theme)
 {
+  gchar *scheme;
+
   g_free (theme->gtk_theme_name);
   theme->gtk_theme_name = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
 
   g_free (theme->gtk_color_scheme);
-  theme->gtk_color_scheme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
+  scheme = gconf_client_get_string (client, COLOR_SCHEME_KEY, NULL);
+
+  if (scheme == NULL || !strcmp (scheme, "")) {
+    g_free (scheme);
+    scheme = gtkrc_get_color_scheme_for_theme (theme->gtk_theme_name);
+  }
+  theme->gtk_color_scheme = scheme;
 
   g_free (theme->metacity_theme_name);
   theme->metacity_theme_name = gconf_client_get_string (client, METACITY_THEME_KEY, NULL);
 
   g_free (theme->icon_theme_name);
   theme->icon_theme_name = gconf_client_get_string (client, ICON_THEME_KEY, NULL);
+
+  g_free (theme->cursor_theme_name);
+
+#ifdef HAVE_XCURSOR
+  theme->cursor_theme_name = gconf_client_get_string (client, CURSOR_THEME_KEY, NULL);
+  theme->cursor_size = gconf_client_get_int (client, CURSOR_SIZE_KEY, NULL);
+#else
+  theme->cursor_theme_name = gconf_client_get_string (client, CURSOR_FONT_KEY, NULL);
+#endif
 }
 
 static gchar *
@@ -193,12 +319,6 @@ theme_is_equal (const GnomeThemeMetaInfo *a, const GnomeThemeMetaInfo *b)
       strcmp (a->gtk_theme_name, b->gtk_theme_name))
     return FALSE;
 
-  a_set = a->gtk_color_scheme && strcmp (a->gtk_color_scheme, "");
-  b_set = b->gtk_color_scheme && strcmp (b->gtk_color_scheme, "");
-  if ((a_set != b_set) ||
-      (a_set && strcmp (a->gtk_color_scheme, b->gtk_color_scheme)))
-    return FALSE;
-
   if (!(a->icon_theme_name && b->icon_theme_name) ||
       strcmp (a->icon_theme_name, b->icon_theme_name))
     return FALSE;
@@ -207,6 +327,16 @@ theme_is_equal (const GnomeThemeMetaInfo *a, const GnomeThemeMetaInfo *b)
       strcmp (a->metacity_theme_name, b->metacity_theme_name))
     return FALSE;
 
+  if (!(a->cursor_theme_name && b->cursor_theme_name) ||
+      strcmp (a->cursor_theme_name, b->cursor_theme_name))
+    return FALSE;
+
+  a_set = a->gtk_color_scheme && strcmp (a->gtk_color_scheme, "");
+  b_set = b->gtk_color_scheme && strcmp (b->gtk_color_scheme, "");
+  if ((a_set != b_set) ||
+      (a_set && !gnome_theme_color_scheme_equal (a->gtk_color_scheme, b->gtk_color_scheme)))
+    return FALSE;
+
   return TRUE;
 }
 
@@ -279,6 +409,147 @@ theme_set_custom_from_theme (const GnomeThemeMetaInfo *info, AppearanceData *dat
 
 /** GUI Callbacks **/
 
+static void
+theme_message_area_response_cb (GtkWidget *w,
+                                gint response_id,
+                                AppearanceData *data)
+{
+  const GnomeThemeMetaInfo *theme;
+
+  theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
+  if (!theme)
+    return;
+
+  switch (response_id)
+  {
+    case RESPONSE_APPLY_BG:
+      gconf_client_set_string (data->client, BACKGROUND_KEY,
+                               theme->background_image, NULL);
+      break;
+
+    case RESPONSE_APPLY_FONT:
+      if (theme->application_font)
+        gconf_client_set_string (data->client, APPLICATION_FONT_KEY,
+                                 theme->application_font, NULL);
+
+      if (theme->desktop_font)
+        gconf_client_set_string (data->client, DESKTOP_FONT_KEY,
+                                 theme->desktop_font, NULL);
+
+      if (theme->monospace_font)
+        gconf_client_set_string (data->client, MONOSPACE_FONT_KEY,
+                                 theme->monospace_font, NULL);
+      break;
+  }
+}
+
+static void
+theme_message_area_update (AppearanceData *data)
+{
+  const GnomeThemeMetaInfo *theme;
+  gboolean show_apply_background = FALSE, show_apply_font = FALSE;
+  const gchar *message;
+  gchar *font;
+
+  theme = theme_get_selected (GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list")), data);
+
+  if (!theme)
+    return;
+
+  if (theme->background_image != NULL) {
+    gchar *background;
+
+    background = gconf_client_get_string (data->client, BACKGROUND_KEY, NULL);
+    show_apply_background =
+        (!background || strcmp (theme->background_image, background) != 0);
+    g_free (background);
+  }
+
+  if (theme->application_font) {
+    font = gconf_client_get_string (data->client, APPLICATION_FONT_KEY, NULL);
+    show_apply_font =
+        (!font || strcmp (theme->application_font, font) != 0);
+    g_free (font);
+  }
+
+  if (!show_apply_font && theme->desktop_font) {
+    font = gconf_client_get_string (data->client, DESKTOP_FONT_KEY, NULL);
+    show_apply_font =
+        (!font || strcmp (theme->application_font, font) != 0);
+    g_free (font);
+  }
+
+  if (!show_apply_font && theme->monospace_font) {
+    font = gconf_client_get_string (data->client, MONOSPACE_FONT_KEY, NULL);
+    show_apply_font =
+        (!font || strcmp (theme->application_font, font) != 0);
+    g_free (font);
+  }
+
+  if (data->theme_message_area == NULL) {
+    GtkWidget *hbox;
+    GtkWidget *icon;
+    GtkWidget *parent;
+
+    if (!show_apply_background && !show_apply_font)
+      return;
+
+    data->theme_message_area = gedit_message_area_new ();
+    gtk_widget_set_no_show_all (data->theme_message_area, TRUE);
+
+    g_signal_connect (data->theme_message_area, "response",
+                      (GCallback) theme_message_area_response_cb, data);
+
+    data->apply_background_button = gedit_message_area_add_button (
+        GEDIT_MESSAGE_AREA (data->theme_message_area),
+        _("Apply Background"),
+        RESPONSE_APPLY_BG);
+    data->apply_font_button = gedit_message_area_add_button (
+        GEDIT_MESSAGE_AREA (data->theme_message_area),
+        _("Apply Font"),
+        RESPONSE_APPLY_FONT);
+
+    data->theme_message_label = gtk_label_new (NULL);
+    gtk_widget_show (data->theme_message_label);
+    gtk_label_set_line_wrap (GTK_LABEL (data->theme_message_label), TRUE);
+    gtk_misc_set_alignment (GTK_MISC (data->theme_message_label), 0.0, 0.5);
+
+    hbox = gtk_hbox_new (FALSE, 9);
+    gtk_widget_show (hbox);
+    icon = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+    gtk_widget_show (icon);
+    gtk_misc_set_alignment (GTK_MISC (icon), 0.5, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), icon, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (hbox), data->theme_message_label, TRUE, TRUE, 0);
+    gedit_message_area_set_contents (GEDIT_MESSAGE_AREA (data->theme_message_area), hbox);
+
+    parent = glade_xml_get_widget (data->xml, "theme_list_vbox");
+    gtk_box_pack_start (GTK_BOX (parent), data->theme_message_area, FALSE, FALSE, 0);
+  }
+
+  if (show_apply_background && show_apply_font) {
+    gtk_widget_show (data->theme_message_area);
+    gtk_widget_show (data->apply_background_button);
+    gtk_widget_show (data->apply_font_button);
+    message = _("The current theme suggests a background and a font.");
+  } else if (show_apply_background) {
+    gtk_widget_show (data->theme_message_area);
+    gtk_widget_show (data->apply_background_button);
+    gtk_widget_hide (data->apply_font_button);
+    message = _("The current theme suggests a background.");
+  } else if (show_apply_font) {
+    gtk_widget_show (data->theme_message_area);
+    gtk_widget_hide (data->apply_background_button);
+    gtk_widget_show (data->apply_font_button);
+    message = _("The current theme suggests a font.");
+  } else {
+    gtk_widget_hide (data->theme_message_area);
+    message = NULL;
+  }
+
+  gtk_label_set_text (GTK_LABEL (data->theme_message_label), message);
+}
+
 static void
 theme_selection_changed_cb (GtkWidget *icon_view, AppearanceData *data)
 {
@@ -304,8 +575,10 @@ theme_selection_changed_cb (GtkWidget *icon_view, AppearanceData *data)
     else
       theme = gnome_theme_meta_info_find (name);
 
-    if (theme)
+    if (theme) {
       gnome_meta_theme_set (theme);
+      theme_message_area_update (data);
+    }
 
     g_free (name);
     g_list_foreach (selection, (GFunc) gtk_tree_path_free, NULL);
@@ -322,10 +595,6 @@ theme_custom_cb (GtkWidget *button, AppearanceData *data)
 {
   GtkWidget *w, *parent;
 
-  /* select the "custom" metatheme */
-  w = glade_xml_get_widget (data->xml, "theme_list");
-  theme_set_custom_from_theme (theme_get_selected (GTK_ICON_VIEW (w), data), data);
-
   w = glade_xml_get_widget (data->xml, "theme_details");
   parent = glade_xml_get_widget (data->xml, "appearance_window");
   gtk_window_set_transient_for (GTK_WINDOW (w), GTK_WINDOW (parent));
@@ -424,9 +693,9 @@ theme_details_changed_cb (AppearanceData *data)
 }
 
 static void
-theme_color_scheme_changed_cb (GObject *settings,
-                               GParamSpec *pspec,
-                               AppearanceData *data)
+theme_setting_changed_cb (GObject *settings,
+                          GParamSpec *pspec,
+                          AppearanceData *data)
 {
   theme_details_changed_cb (data);
 }
@@ -440,20 +709,6 @@ theme_gconf_changed (GConfClient *client,
   theme_details_changed_cb (data);
 }
 
-static void
-theme_postinit (GtkIconView *icon_view, AppearanceData *data)
-{
-  /* connect to individual gconf changes; we only do that now to make sure
-   * we don't receive any signals before the widgets have been realized */
-  gconf_client_add_dir (data->client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_NONE, NULL);
-  gconf_client_add_dir (data->client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_NONE, NULL);
-  gconf_client_notify_add (data->client, GTK_THEME_KEY, (GConfClientNotifyFunc) theme_gconf_changed, data, NULL, NULL);
-  gconf_client_notify_add (data->client, METACITY_THEME_KEY, (GConfClientNotifyFunc) theme_gconf_changed, data, NULL, NULL);
-  gconf_client_notify_add (data->client, ICON_THEME_KEY, (GConfClientNotifyFunc) theme_gconf_changed, data, NULL, NULL);
-
-  g_signal_connect (gtk_settings_get_default (), "notify::gtk-color-scheme", (GCallback) theme_color_scheme_changed_cb, data);
-}
-
 static gint
 theme_list_sort_func (GnomeThemeMetaInfo *a,
                       GnomeThemeMetaInfo *b)
@@ -494,6 +749,47 @@ theme_store_sort_func (GtkTreeModel *model,
   return rc;
 }
 
+static void
+theme_drag_data_received_cb (GtkWidget *widget,
+                             GdkDragContext *context,
+                             gint x, gint y,
+                             GtkSelectionData *selection_data,
+                             guint info, guint time,
+                             AppearanceData *data)
+{
+  GList *uris;
+
+  if (!(info == TARGET_URI_LIST || info == TARGET_NS_URL))
+    return;
+
+  uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
+
+  if (uris != NULL && uris->data != NULL) {
+    GnomeVFSURI *uri = (GnomeVFSURI *) uris->data;
+    gchar *filename;
+
+    if (gnome_vfs_uri_is_local (uri))
+      filename = gnome_vfs_unescape_string (gnome_vfs_uri_get_path (uri), G_DIR_SEPARATOR_S);
+    else
+      filename = gnome_vfs_unescape_string (gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE), G_DIR_SEPARATOR_S);
+
+    gnome_vfs_uri_list_unref (uris);
+
+    gnome_theme_install_from_uri (filename,
+        GTK_WINDOW (glade_xml_get_widget (data->xml, "appearance_window")));
+    g_free (filename);
+  }
+}
+
+static void
+background_or_font_changed (GConfEngine *conf,
+                            guint cnxn_id,
+                            GConfEntry *entry,
+                            AppearanceData *data)
+{
+  theme_message_area_update (data);
+}
+
 void
 themes_init (AppearanceData *data)
 {
@@ -502,19 +798,21 @@ themes_init (AppearanceData *data)
   GtkListStore *theme_store;
   GtkTreeModel *sort_model;
   GnomeThemeMetaInfo *meta_theme = NULL;
+  GtkIconView *icon_view;
+  GtkCellRenderer *renderer;
+  GtkSettings *settings;
 
   /* initialise some stuff */
   gnome_theme_init (NULL);
   gnome_wm_manager_init ();
 
   data->theme_save_dialog = NULL;
+  data->theme_message_area = NULL;
   data->theme_custom = gnome_theme_meta_info_new ();
   data->theme_icon = gdk_pixbuf_new_from_file (GNOMECC_PIXMAP_DIR "/theme-thumbnailing.png", NULL);
   data->theme_store = theme_store =
       gtk_list_store_new (NUM_COLS, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
 
-  del_button = glade_xml_get_widget (data->xml, "theme_delete");
-
   /* set up theme list */
   theme_list = gnome_theme_meta_info_find_all ();
   gnome_theme_info_register_theme_change ((ThemeChangedCallback) theme_changed_on_disk_cb, data);
@@ -554,14 +852,30 @@ themes_init (AppearanceData *data)
   g_list_foreach (theme_list, (GFunc) theme_thumbnail_generate, data);
   g_list_free (theme_list);
 
-  w = glade_xml_get_widget (data->xml, "theme_list");
+  icon_view = GTK_ICON_VIEW (glade_xml_get_widget (data->xml, "theme_list"));
+
+  renderer = cell_renderer_wallpaper_new ();
+  g_object_set (renderer, "xpad", 5, "ypad", 5,
+                          "xalign", 0.5, "yalign", 1.0, NULL);
+  gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (icon_view), renderer, FALSE);
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (icon_view), renderer,
+                                  "pixbuf", COL_THUMBNAIL, NULL);
+
+  renderer = gtk_cell_renderer_text_new ();
+  g_object_set (renderer, "alignment", PANGO_ALIGN_CENTER,
+			  "wrap-mode", PANGO_WRAP_WORD_CHAR,
+			  "xalign", 0.5, "yalign", 0.0, NULL);
+  gtk_cell_layout_pack_end (GTK_CELL_LAYOUT (icon_view), renderer, FALSE);
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (icon_view), renderer,
+                                  "markup", COL_LABEL, NULL);
+
   sort_model = gtk_tree_model_sort_new_with_model (GTK_TREE_MODEL (theme_store));
-  gtk_icon_view_set_model (GTK_ICON_VIEW (w), GTK_TREE_MODEL (sort_model));
   gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (sort_model), COL_LABEL, theme_store_sort_func, NULL, NULL);
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (sort_model), COL_LABEL, GTK_SORT_ASCENDING);
-  g_signal_connect (w, "selection-changed", (GCallback) theme_selection_changed_cb, data);
-  g_signal_connect_after (w, "realize", (GCallback) theme_select_name, meta_theme->name);
-  g_signal_connect_after (w, "realize", (GCallback) theme_postinit, data);
+  gtk_icon_view_set_model (icon_view, GTK_TREE_MODEL (sort_model));
+
+  g_signal_connect (icon_view, "selection-changed", (GCallback) theme_selection_changed_cb, data);
+  g_signal_connect_after (icon_view, "realize", (GCallback) theme_select_name, meta_theme->name);
 
   w = glade_xml_get_widget (data->xml, "theme_install");
   gtk_button_set_image (GTK_BUTTON (w),
@@ -578,11 +892,30 @@ themes_init (AppearanceData *data)
                         gtk_image_new_from_stock (GTK_STOCK_EDIT, GTK_ICON_SIZE_BUTTON));
   g_signal_connect (w, "clicked", (GCallback) theme_custom_cb, data);
 
+  del_button = glade_xml_get_widget (data->xml, "theme_delete");
   g_signal_connect (del_button, "clicked", (GCallback) theme_delete_cb, data);
 
-  if (is_locked_down (data->client)) {
-    /* FIXME: determine what needs disabling */
-  }
+  w = glade_xml_get_widget (data->xml, "theme_vbox");
+  gtk_drag_dest_set (w, GTK_DEST_DEFAULT_ALL,
+		     drop_types, G_N_ELEMENTS (drop_types),
+		     GDK_ACTION_COPY | GDK_ACTION_LINK | GDK_ACTION_MOVE);
+  g_signal_connect (w, "drag-data-received", (GCallback) theme_drag_data_received_cb, data);
+  if (is_locked_down (data->client))
+    gtk_widget_set_sensitive (w, FALSE);
+
+  /* listen to gconf changes, too */
+  gconf_client_add_dir (data->client, "/apps/metacity/general", GCONF_CLIENT_PRELOAD_NONE, NULL);
+  gconf_client_add_dir (data->client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_NONE, NULL);
+  gconf_client_notify_add (data->client, METACITY_THEME_KEY, (GConfClientNotifyFunc) theme_gconf_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, BACKGROUND_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, APPLICATION_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, DESKTOP_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
+  gconf_client_notify_add (data->client, MONOSPACE_FONT_KEY, (GConfClientNotifyFunc) background_or_font_changed, data, NULL, NULL);
+
+  settings = gtk_settings_get_default ();
+  g_signal_connect (settings, "notify::gtk-color-scheme", (GCallback) theme_setting_changed_cb, data);
+  g_signal_connect (settings, "notify::gtk-theme-name", (GCallback) theme_setting_changed_cb, data);
+  g_signal_connect (settings, "notify::gtk-icon-theme-name", (GCallback) theme_setting_changed_cb, data);
 }
 
 void
